package com.github.eildosa.app;

/**
 * 
 * If this does say : "configure java.library.path"
 * right clic on this file, choose Run as>run configuration
 * clic on the argument tab and then add this : -Djava.library.path=target/natives
 * in the box under VM arguments, then apply
 * 
 * This is the core of the application.
 * Be carefull, IF YOU CHANGE THE NAME OF THIS CLASS then you must also change
 * in the pom.xml under <main.class>XXXXX</main.class> the name of the class
 * 
 * Because the name of this class is used in the assembly.xml
 * 
 * @author athongvan
 */


import org.lwjgl.input.Keyboard;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.util.glu.GLU;


public class AppTest {
    
    private final String windowTitle = "Colorful rotating cube";
    private DisplayMode displayMode;
    private boolean done = false;
    private float x, y, z;
   
    public static void main(String args[]) {
        AppTest t6 = new AppTest();
        t6.run();
    }

    /**
     * MAIN loop
     */
    public void run() {
        try {
            init();
            while (!done) {
                readAndAct();
                render();
                Display.update();
            }
            Destroy();
        }
        catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }
    }

    /**
     * Every update must be done here, keyboard scanning, mouse input, IA, etc 
     * 
     */
    private void readAndAct() {
    	//exit if escape is pressed
        if(Keyboard.isKeyDown(Keyboard.KEY_ESCAPE)) {
            done = true;
        }
        //exit if the red cross on the windows is cliqued
        if(Display.isCloseRequested()) {             
            done = true;
        }

    }
    

    
    
    

    /**
     * here everything is being drawed
     * le "rendering"
     * @return boolean : true if everything went right
     */
    private boolean render() {
    	GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT); // Clear The Screen And The Depth Buffer
    	GL11.glLoadIdentity(); // Reset The Current Modelview Matrix
    	GL11.glTranslatef(0.0f, 0.0f, -6.0f);

    	Draw.cubeHereColorfulUsingTriangle(x, y, z, 1.0f);
    	
    	x += 0.035f;
    	y += 0.02f*2f;
    	z += 0.05f*1f;
        return true;
    }

    /**
     * Make the window
     * @throws Exception Throws Window.create() exception.
     */
    private void createWindow() throws Exception {
    	//demande tous les mode d'affichage disponible
        DisplayMode d[] = Display.getAvailableDisplayModes();
        //parcour tous les mode retourné et selectionne celui ou
        //hauteur = 480, largeur = 640, et ou on a 32 bits par pixel
        for (int i = 0; i < d.length; i++) {
            if (d[i].getWidth() == 640
                && d[i].getHeight() == 480
                && d[i].getBitsPerPixel() == 32) {
                displayMode = d[i];
                break;
            }
        }
        Display.setDisplayMode(displayMode);
        Display.setTitle(windowTitle);
        Display.create();
    }

    /**
     * Faire toute les initialisation ici, comme le clavier et openGL
     */
    private void init() throws Exception {
        createWindow();
        initGL();
    }
    

    

    /**
     * Init OpenGL
     *
     */
    private void initGL() {
    	
        GL11.glEnable(GL11.GL_TEXTURE_2D);//texture mapping 
        GL11.glShadeModel(GL11.GL_SMOOTH);//smooth shading 
        GL11.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);//couleur de fond noire
        GL11.glClearDepth(1.0);//setup du buffer de profondeur 
        GL11.glEnable(GL11.GL_DEPTH_TEST);//permet le test de profondeur 
        GL11.glDepthFunc(GL11.GL_LEQUAL); //le type de test de profondeur a faire
        GL11.glMatrixMode(GL11.GL_PROJECTION);//selectrione la matrice de projection    
        GL11.glLoadIdentity();//reset la matrice de projection
        // Calcul l'aspect ratio de la fenetre
        GLU.gluPerspective(
          45.0f,
          (float) displayMode.getWidth() / (float) displayMode.getHeight(),
          0.1f,
          100.0f);
          
          GL11.glMatrixMode(GL11.GL_MODELVIEW);//selectionne la modelview Matrix
          // met les melleur parametre
          GL11.glHint(GL11.GL_PERSPECTIVE_CORRECTION_HINT, GL11.GL_NICEST);
    }

    /**
     * Cleaning
     */
    private void Destroy() {
        Display.destroy();
    }
    
    /**
     * Used to banch time difference betwen different drawing function
     */
    private void benchmark(){
    	long start = System.currentTimeMillis();
    	for(int i=0; i<10000; i++){
    		Draw.cubeHereColorfulUsingTriangle(x, y, z, 1.0f);
    	}
    	long end = System.currentTimeMillis();
    	System.out.println("Execution time was "+(end-start)+" ms. for drawing with 2 triangle per face");
    	
    	long start2 = System.currentTimeMillis();
    	for(int i=0; i<10000; i++){
    		Draw.cubeHereColorful(x, y, z, 1.0f);
    	}
    	long end2 = System.currentTimeMillis();
    	System.out.println("Execution time was "+(end2-start2)+" ms. for drawing with 1 square per face");
    }

}




